@@ -12,6 +12,7 @@
  */
 
 #include <config.h>
+#include <errno.h>
 #include <features.h>
 #include <fcntl.h>
 #include <malloc.h>
@@ -73,6 +74,8 @@ int main(int argc, char **argv){
     off_t		offset = 0, sf = 0;	/// seek postition, lseek result
     int			start, res, stop;	/// start, range, stop number for progress bar
     unsigned long long	total_write = 0;	/// the copied size 
+    unsigned long long	needed_size = 0;	/// the copied size 
+    unsigned long long	needed_mem  = 0;	/// the copied size 
     char		bitmagic[8] = "BiTmAgIc";// only for check postition
     char		bitmagic_r[8];		/// read magic string from image
     int			cmp;			/// compare magic string
@@ -84,6 +87,14 @@ int main(int argc, char **argv){
     char*		crc_buffer;		/// buffer data for malloc crc code
     int			done = 0;
     int			s_count = 0;
+    int			rescue_num = 0;
+    char *bad_sectors_warning_msg =
+    "*************************************************************************\n"
+    "* WARNING: The disk has bad sector. This means physical damage on the   *\n"
+    "* disk surface caused by deterioration, manufacturing faults or other   *\n"
+    "* reason. The reliability of the disk may stay stable or degrade fast.  *\n"
+    "* Use the --rescue option to efficiently save as much data as possible! *\n"
+    "*************************************************************************\n";
 
     progress_bar	prog;			/// progress_bar structure defined in progress.h
     cmd_opt		opt;			/// cmd_opt structure defined in partclone.h
@@ -95,11 +106,18 @@ int main(int argc, char **argv){
      */
     parse_options(argc, argv, &opt);
 
+    /**
+     * if "-d / --debug" given
+     * open debug file in "/var/log/partclone.log" for log message 
+     */
+    debug = opt.debug;
+    //if(opt.debug)
+	open_log();
+
     if (geteuid() != 0)
 	log_mesg(0, 1, 1, debug, "You are not logged as root. You may have \"access denied\" errors when working.\n"); 
     else
-	log_mesg(0, 0, 0, debug, "UID is root.\n");
-
+	log_mesg(1, 0, 0, debug, "UID is root.\n");
 
     /**
      * open source and target 
@@ -110,15 +128,14 @@ int main(int argc, char **argv){
     source = opt.source;
     target = opt.target;
     dfr = open_source(source, &opt);
-    dfw = open_target(target, &opt);
+    if (dfr == -1) {
+	log_mesg(0, 1, 1, debug, "Erro EXIT.\n");
+    }
 
-    /**
-     * if "-d / --debug" given
-     * open debug file in "/var/log/partclone.log" for log message 
-     */
-    debug = opt.debug;
-    //if(opt.debug)
-	open_log();
+    dfw = open_target(target, &opt);
+    if (dfw == -1) {
+	log_mesg(0, 1, 1, debug, "Error Exit.\n");
+    }
 
     /**
      * get partition information like super block, image_head, bitmap
@@ -126,23 +143,35 @@ int main(int argc, char **argv){
      */
     if (opt.clone){
 
-	log_mesg(0, 0, 0, debug, "Initial image hdr: get Super Block from partition\n");
+	log_mesg(1, 0, 0, debug, "Initial image hdr - get Super Block from partition\n");
 	
 	/// get Super Block information from partition
         initial_image_hdr(source, &image_hdr);
 
+        /// check memory size
+        if (check_mem_size(image_hdr, opt, &needed_mem) == -1)
+            log_mesg(0, 1, 1, debug, "Ther is no enough free memory, partclone suggests you should have %i bytes memory\n", needed_mem);
+            
 	memcpy(image_hdr.version, IMAGE_VERSION, VERSION_SIZE);
 
 	/// alloc a memory to restore bitmap
 	bitmap = (char*)malloc(sizeof(char)*image_hdr.totalblock);
+        if(bitmap == NULL){
+            log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
+        }
 	
-	log_mesg(0, 0, 0, debug, "initial main bitmap pointer %i\n", bitmap);
-	log_mesg(0, 0, 0, debug, "Initial image hdr: read bitmap table\n");
+	log_mesg(2, 0, 0, debug, "initial main bitmap pointer %i\n", bitmap);
+	log_mesg(1, 0, 0, debug, "Initial image hdr - read bitmap table\n");
 
 	/// read and check bitmap from partition
+	log_mesg(1, 0, 0, debug, "Calculating bitmap ...");
 	readbitmap(source, image_hdr, bitmap);
+	log_mesg(1, 0, 0, debug, "done\n");
 
-	log_mesg(0, 0, 0, debug, "check main bitmap pointer %i\n", bitmap);
+	needed_size = (unsigned long long)(((image_hdr.block_size+sizeof(unsigned long))*image_hdr.usedblocks)+sizeof(image_hdr)+sizeof(char)*image_hdr.totalblock);
+	check_free_space(&dfw, needed_size);
+
+	log_mesg(2, 0, 0, debug, "check main bitmap pointer %i\n", bitmap);
 
 	/*
 	sf = lseek(dfw, 0, SEEK_SET);
@@ -163,34 +192,72 @@ int main(int argc, char **argv){
 
     } else if (opt.restore){
 
-	log_mesg(0, 0, 0, debug, "restore image hdr: get image_head from image file\n");
+	log_mesg(1, 0, 0, debug, "restore image hdr - get image_head from image file\n");
         /// get image information from image file
 	restore_image_hdr(&dfr, &opt, &image_hdr);
+        
+        /// check memory size
+        if (check_mem_size(image_hdr, opt, &needed_mem) == -1)
+            log_mesg(0, 1, 1, debug, "Ther is no enough free memory, partclone suggests you should have %i bytes memory\n", needed_mem);
 
 	/// alloc a memory to restore bitmap
 	bitmap = (char*)malloc(sizeof(char)*image_hdr.totalblock);
+        if(bitmap == NULL){
+            log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
+        }
 
 	/// check the image magic
 	if (memcmp(image_hdr.magic, IMAGE_MAGIC, IMAGE_MAGIC_SIZE) != 0)
-	    log_mesg(0, 1, 1, debug, "This is nor partclone image.\n");
+	    log_mesg(0, 1, 1, debug, "This is not partclone image.\n");
 
 	/// check the file system
 	if (memcmp(image_hdr.fs, FS, FS_MAGIC_SIZE) != 0)
-	    log_mesg(0, 1, 1, debug, "The image file system error.\n");
+	    log_mesg(0, 1, 1, debug, "Partclone can't restore partition from the image because the file system is different.\n");
 
-	log_mesg(0, 0, 0, debug, "initial main bitmap pointer %lli\n", bitmap);
-	log_mesg(0, 0, 0, debug, "Initial image hdr: read bitmap table\n");
+	log_mesg(2, 0, 0, debug, "initial main bitmap pointer %lli\n", bitmap);
+	log_mesg(1, 0, 0, debug, "Initial image hdr - read bitmap table\n");
 
 	/// read and check bitmap from image file
+	log_mesg(1, 0, 0, debug, "Calculating bitmap ...\n");
 	get_image_bitmap(&dfr, opt, image_hdr, bitmap);
 
 	/// check the dest partition size.
 	check_size(&dfw, image_hdr.device_size);
 
-	log_mesg(0, 0, 0, debug, "check main bitmap pointer %i\n", bitmap);
+	log_mesg(2, 0, 0, debug, "check main bitmap pointer %i\n", bitmap);
+    } else if (opt.dd){
+	log_mesg(1, 0, 0, debug, "Initial image hdr - get Super Block from partition\n");
+
+	/// get Super Block information from partition
+	initial_image_hdr(source, &image_hdr);
+
+        /// check memory size
+        if (check_mem_size(image_hdr, opt, &needed_mem) == -1)
+            log_mesg(0, 1, 1, debug, "Ther is no enough free memory, partclone suggests you should have %i bytes memory\n", needed_mem);
+
+	memcpy(image_hdr.version, IMAGE_VERSION, VERSION_SIZE);
+
+	/// alloc a memory to restore bitmap
+	bitmap = (char*)malloc(sizeof(char)*image_hdr.totalblock);
+        if(bitmap == NULL){
+            log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
+        }
+
+	log_mesg(2, 0, 0, debug, "initial main bitmap pointer %i\n", bitmap);
+	log_mesg(1, 0, 0, debug, "Initial image hdr - read bitmap table\n");
+
+	/// read and check bitmap from partition
+	log_mesg(1, 0, 0, debug, "Calculating bitmap ...\n");
+	readbitmap(source, image_hdr, bitmap);
+
+	log_mesg(2, 0, 0, debug, "check main bitmap pointer %i\n", bitmap);
     }
 
-    log_mesg(0, 0, 0, debug, "print image_head\n");
+    log_mesg(1, 0, 0, debug, "print image_head\n");
+
+    /// print option to log file
+    if (debug)
+	print_opt(opt);
 
     /// print image_head
     print_image_hdr_info(image_hdr, opt);
@@ -201,7 +268,7 @@ int main(int argc, char **argv){
     start = 0;				/// start number of progress bar
     stop = (int)image_hdr.usedblocks;	/// get the end of progress number, only used block
     res = 100;				/// the end of progress number
-    log_mesg(0, 0, 0, debug, "Initial Progress bar\n");
+    log_mesg(1, 0, 0, debug, "Initial Progress bar\n");
     /// Initial progress bar
     progress_init(&prog, start, stop, res, (int)image_hdr.block_size);
     copied = 1;				/// initial number is 1
@@ -223,7 +290,7 @@ int main(int argc, char **argv){
 
 	/// read data from the first block and log the offset
 	sf = lseek(dfr, 0, SEEK_SET);
-	log_mesg(0, 0, 0, debug, "seek %lli for reading data string\n",sf);
+	log_mesg(1, 0, 0, debug, "seek %lli for reading data string\n",sf);
         if (sf == (off_t)-1)
 	    log_mesg(0, 1, 1, debug, "seek set %lli\n", sf);
 
@@ -235,36 +302,52 @@ int main(int argc, char **argv){
 	    
 	    r_size = 0;
 	    w_size = 0;
-	    log_mesg(0, 0, 0, debug, "block_id=%lli, ",block_id);
 
 	    if((image_hdr.totalblock - 1 ) == block_id) 
 		done = 1;
 
 	    if (bitmap[block_id] == 1){
 		/// if the block is used
-
-		log_mesg(0, 0, 0, debug, "bitmap=%i, ",bitmap[block_id]);
+		log_mesg(1, 0, 0, debug, "block_id=%lli, ",block_id);
+		log_mesg(1, 0, 0, debug, "bitmap=%i, ",bitmap[block_id]);
 
 		offset = (off_t)(block_id * image_hdr.block_size);
 		//sf = lseek(dfr, offset, SEEK_SET);
                 //if (sf == (off_t)-1)
                 //    log_mesg(0, 1, 1, debug, "seek error %lli errno=%i\n", (long long)offset, (int)errno);
         	buffer = (char*)malloc(image_hdr.block_size); ///alloc a memory to copy data
+                if(buffer == NULL){
+                    log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
+                }
         	
 		/// read data from source to buffer
 		r_size = read_all(&dfr, buffer, image_hdr.block_size, &opt);
-		log_mesg(0, 0, 0, debug, "bs=%i and r=%i, ",image_hdr.block_size, r_size);
-		if (r_size != (int)image_hdr.block_size)
+		log_mesg(1, 0, 0, debug, "bs=%i and r=%i, ",image_hdr.block_size, r_size);
+		if (r_size != (int)image_hdr.block_size){
+
+		    if ((r_size == -1) && (errno == EIO)){
+			if (opt.rescue){
+			    for (rescue_num = 0; rescue_num < image_hdr.block_size; rescue_num += SECTOR_SIZE)
+				 rescue_sector(&dfr, buffer + rescue_num, &opt);
+			}else
+                             log_mesg(0, 1, 1, debug, "%s", bad_sectors_warning_msg);
+			
+		    }
+
 		    log_mesg(0, 1, 1, debug, "read error %i \n", r_size);
+		}
         	
 		/// write buffer to target
 		w_size = write_all(&dfw, buffer, image_hdr.block_size, &opt);
-		log_mesg(0, 0, 0, debug, "bs=%i and w=%i, ",image_hdr.block_size, w_size);
+		log_mesg(2, 0, 0, debug, "bs=%i and w=%i, ",image_hdr.block_size, w_size);
 		if (w_size != (int)image_hdr.block_size)
 		    log_mesg(0, 1, 1, debug, "write error %i \n", w_size);
 
 		/// generate crc32 code and write it.
         	crc_buffer = (char*)malloc(sizeof(unsigned long)); ///alloc a memory to copy data
+                if(crc_buffer == NULL){
+                    log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
+                }
 		crc = crc32(crc, buffer, w_size);
 		memcpy(crc_buffer, &crc, sizeof(unsigned long));
 		c_size = write_all(&dfw, crc_buffer, sizeof(unsigned long), &opt);
@@ -277,15 +360,17 @@ int main(int argc, char **argv){
         	
 		copied++;					/// count copied block
 		total_write += (unsigned long long)(w_size);	/// count copied size
-		log_mesg(0, 0, 0, debug, "total=%lli, ", total_write);
+		log_mesg(1, 0, 0, debug, "total=%lli, ", total_write);
 		
 		/// read or write error
 		if (r_size != w_size)
 		    log_mesg(0, 1, 1, debug, "read and write different\n");
+		log_mesg(1, 0, 0, debug, "end\n");
             } else {
 		/// if the block is not used, I just skip it.
+		log_mesg(2, 0, 0, debug, "block_id=%lli, ",block_id);
         	sf = lseek(dfr, image_hdr.block_size, SEEK_CUR);
-		log_mesg(0, 0, 0, debug, "skip seek=%lli, ",sf);
+		log_mesg(2, 0, 0, debug, "skip seek=%lli, ",sf);
                 if (sf == (off_t)-1)
                     log_mesg(0, 1, 1, debug, "clone seek error %lli errno=%i\n", (long long)offset, (int)errno);
 	    
@@ -294,8 +379,8 @@ int main(int argc, char **argv){
 		    progress_update(&prog, copied, done);
 		    s_count = 0;
 		}
+		log_mesg(2, 0, 0, debug, "end\n");
 	    }
-	    log_mesg(0, 0, 0, debug, "end\n");
         } /// end of for    
 	sync_data(dfw, &opt);	
     
@@ -312,7 +397,7 @@ int main(int argc, char **argv){
 
 	/// seek to the first
         sf = lseek(dfw, 0, SEEK_SET);
-	log_mesg(0, 0, 0, debug, "seek %lli for writing dtat string\n",sf);
+	log_mesg(1, 0, 0, debug, "seek %lli for writing dtat string\n",sf);
 	if (sf == (off_t)-1)
 	    log_mesg(0, 1, 1, debug, "seek set %lli\n", sf);
     
@@ -327,70 +412,155 @@ int main(int argc, char **argv){
 	/// start restore image file to partition
         for( block_id = 0; block_id < image_hdr.totalblock; block_id++ ){
 
-	r_size = 0;
-	w_size = 0;
-        log_mesg(0, 0, 0, debug, "block_id=%lli, ",block_id);
+	    r_size = 0;
+	    w_size = 0;
 
-	if((block_id + 1) == image_hdr.totalblock) 
-	    done = 1;
+	    if((block_id + 1) == image_hdr.totalblock) 
+		done = 1;
 
-    	if (bitmap[block_id] == 1){ 
-	    /// The block is used
-	    log_mesg(0, 0, 0, debug, "bitmap=%i, ",bitmap[block_id]);
+	    if (bitmap[block_id] == 1){ 
+		/// The block is used
+		log_mesg(2, 0, 0, debug, "block_id=%lli, ",block_id);
+		log_mesg(1, 0, 0, debug, "bitmap=%i, ",bitmap[block_id]);
 
-	    offset = (off_t)(block_id * image_hdr.block_size);
-	    //sf = lseek(dfw, offset, SEEK_SET);
-            //if (sf == (off_t)-1)
-            //    log_mesg(0, 1, 1, debug, "seek error %lli errno=%i\n", (long long)offset, (int)errno);
-	    buffer = (char*)malloc(image_hdr.block_size); ///alloc a memory to copy data
-	    r_size = read_all(&dfr, buffer, image_hdr.block_size, &opt);
-	    log_mesg(0, 0, 0, debug, "bs=%i and r=%i, ",image_hdr.block_size, r_size);
-	    if (r_size <0)
-		log_mesg(0, 1, 1, debug, "read errno = %i \n", errno);
+		offset = (off_t)(block_id * image_hdr.block_size);
+		//sf = lseek(dfw, offset, SEEK_SET);
+		//if (sf == (off_t)-1)
+		//    log_mesg(0, 1, 1, debug, "seek error %lli errno=%i\n", (long long)offset, (int)errno);
+		buffer = (char*)malloc(image_hdr.block_size); ///alloc a memory to copy data
+                if(buffer == NULL){
+                    log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
+                }
+		r_size = read_all(&dfr, buffer, image_hdr.block_size, &opt);
+		log_mesg(1, 0, 0, debug, "bs=%i and r=%i, ",image_hdr.block_size, r_size);
+		if (r_size <0)
+		    log_mesg(0, 1, 1, debug, "read errno = %i \n", errno);
 
-	    /// write block from buffer to partition
-	    w_size = write_all(&dfw, buffer, image_hdr.block_size, &opt);
-	    log_mesg(0, 0, 0, debug, "bs=%i and w=%i, ",image_hdr.block_size, w_size);
-	    if (w_size != (int)image_hdr.block_size)
-		log_mesg(0, 1, 1, debug, "write error %i \n", w_size);
+		/// write block from buffer to partition
+		w_size = write_all(&dfw, buffer, image_hdr.block_size, &opt);
+		log_mesg(1, 0, 0, debug, "bs=%i and w=%i, ",image_hdr.block_size, w_size);
+		if (w_size != (int)image_hdr.block_size)
+		    log_mesg(0, 1, 1, debug, "write error %i \n", w_size);
 
-	    /// read crc32 code and check it.
-	    crc_ck = crc32(crc_ck, buffer, r_size);
-            crc_buffer = (char*)malloc(sizeof(unsigned long)); ///alloc a memory to copy data
-	    c_size = read_all(&dfr, crc_buffer, sizeof(unsigned long), &opt);
-	    memcpy(&crc, crc_buffer, sizeof(unsigned long));
-	    if (memcmp(&crc, &crc_ck, sizeof(unsigned long)) != 0)
-		log_mesg(0, 1, 1, debug, "CRC Check  error\n OrigCRC:0x%08lX, DestCRC:0x%08lX", crc, crc_ck);
+		/// read crc32 code and check it.
+		crc_ck = crc32(crc_ck, buffer, r_size);
+		crc_buffer = (char*)malloc(sizeof(unsigned long)); ///alloc a memory to copy data
+                if(crc_buffer == NULL){
+                    log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
+                }
+		c_size = read_all(&dfr, crc_buffer, sizeof(unsigned long), &opt);
+		memcpy(&crc, crc_buffer, sizeof(unsigned long));
+		if (memcmp(&crc, &crc_ck, sizeof(unsigned long)) != 0)
+		    log_mesg(0, 1, 1, debug, "CRC Check  error\n OrigCRC:0x%08lX, DestCRC:0x%08lX", crc, crc_ck);
 
-	    /// free buffer
-	    free(buffer);
-	    free(crc_buffer);
+		/// free buffer
+		free(buffer);
+		free(crc_buffer);
 
-	    progress_update(&prog, copied, done);
-
-       	    copied++;					/// count copied block
-	    total_write += (unsigned long long) w_size;	/// count copied size
-
-	    /// read or write error
-	    //if ((r_size != w_size) || (r_size != image_hdr.block_size))
-	    //	log_mesg(0, 1, 1, debug, "read and write different\n");
-       	} else {
-
-	    /// if the block is not used, I just skip it.
-	    sf = lseek(dfw, image_hdr.block_size, SEEK_CUR);
-	    log_mesg(0, 0, 0, debug, "seek=%lli, ",sf);
-	    if (sf == (off_t)-1)
-		log_mesg(0, 1, 1, debug, "seek error %lli errno=%i\n", (long long)offset, (int)errno);
-	    s_count++;
-	    if ((s_count >=100) || (done == 1)){
 		progress_update(&prog, copied, done);
-		s_count = 0;
+
+		copied++;					/// count copied block
+		total_write += (unsigned long long) w_size;	/// count copied size
+
+		/// read or write error
+		//if ((r_size != w_size) || (r_size != image_hdr.block_size))
+		//	log_mesg(0, 1, 1, debug, "read and write different\n");
+		log_mesg(1, 0, 0, debug, "end\n");
+	    } else {
+
+		/// if the block is not used, I just skip it.
+		log_mesg(2, 0, 0, debug, "block_id=%lli, ",block_id);
+		sf = lseek(dfw, image_hdr.block_size, SEEK_CUR);
+		log_mesg(2, 0, 0, debug, "seek=%lli, ",sf);
+		if (sf == (off_t)-1)
+		    log_mesg(0, 1, 1, debug, "seek error %lli errno=%i\n", (long long)offset, (int)errno);
+		s_count++;
+		if ((s_count >=100) || (done == 1)){
+		    progress_update(&prog, copied, done);
+		    s_count = 0;
+		}
+		log_mesg(2, 0, 0, debug, "end\n");
 	    }
-	}
-	log_mesg(0, 0, 0, debug, "end\n");
 
     	} // end of for
 	sync_data(dfw, &opt);	
+    } else if (opt.dd){
+	sf = lseek(dfr, 0, SEEK_SET);
+	log_mesg(1, 0, 0, debug, "seek %lli for reading data string\n",sf);
+	if (sf == (off_t)-1)
+	    log_mesg(0, 1, 1, debug, "seek set %lli\n", sf);
+
+	log_mesg(0, 0, 0, debug, "Total block %i\n", image_hdr.totalblock);
+
+	/// start clone partition to image file
+        for( block_id = 0; block_id < image_hdr.totalblock; block_id++ ){
+	    r_size = 0;
+	    w_size = 0;
+
+	    if((image_hdr.totalblock - 1 ) == block_id)
+		done = 1;
+
+	    if (bitmap[block_id] == 1){
+		/// if the block is used
+
+		log_mesg(1, 0, 0, debug, "block_id=%lli, ",block_id);
+		log_mesg(1, 0, 0, debug, "bitmap=%i, ",bitmap[block_id]);
+		offset = (off_t)(block_id * image_hdr.block_size);
+		buffer = (char*)malloc(image_hdr.block_size); ///alloc a memory to copy data
+                if(buffer == NULL){
+                    log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
+                }
+		/// read data from source to buffer
+		r_size = read_all(&dfr, buffer, image_hdr.block_size, &opt);
+		log_mesg(1, 0, 0, debug, "bs=%i and r=%i, ",image_hdr.block_size, r_size);
+		if (r_size != (int)image_hdr.block_size){
+		    if ((r_size == -1) && (errno == EIO)){
+			if (opt.rescue){
+			    for (rescue_num = 0; rescue_num < image_hdr.block_size; rescue_num += SECTOR_SIZE)
+				rescue_sector(&dfr, buffer + rescue_num, &opt);
+			}else
+			    log_mesg(0, 1, 1, debug, "%s", bad_sectors_warning_msg);
+
+		    }
+
+		    log_mesg(0, 1, 1, debug, "read error %i \n", r_size);
+		}
+
+		/// write buffer to target
+		w_size = write_all(&dfw, buffer, image_hdr.block_size, &opt);
+		log_mesg(1, 0, 0, debug, "bs=%i and w=%i, ",image_hdr.block_size, w_size);
+		if (w_size != (int)image_hdr.block_size)
+		    log_mesg(0, 1, 1, debug, "write error %i \n", w_size);
+
+		/// free buffer
+		free(buffer);
+		progress_update(&prog, copied, done);
+		copied++;                                       /// count copied block
+		total_write += (unsigned long long)(w_size);    /// count copied size
+		log_mesg(1, 0, 0, debug, "total=%lli, ", total_write);
+		/// read or write error
+		if (r_size != w_size)
+		    log_mesg(0, 1, 1, debug, "read and write different\n");
+		log_mesg(1, 0, 0, debug, "end\n");
+	    } else {
+		/// if the block is not used, I just skip it.
+		log_mesg(2, 0, 0, debug, "block_id=%lli, ",block_id);
+		sf = lseek(dfr, image_hdr.block_size, SEEK_CUR);
+		log_mesg(2, 0, 0, debug, "skip source seek=%lli, ",sf);
+		sf = lseek(dfw, image_hdr.block_size, SEEK_CUR);
+		log_mesg(2, 0, 0, debug, "skip target seek=%lli, ",sf);
+		if (sf == (off_t)-1)
+		    log_mesg(0, 1, 1, debug, "clone seek error %lli errno=%i\n", (long long)offset, (int)errno);
+		s_count++;
+		if ((s_count >=100) || (done == 1)){
+		    progress_update(&prog, copied, done);
+		     s_count = 0;
+		}
+
+		log_mesg(2, 0, 0, debug, "end\n");
+	    }
+	} /// end of for
+	sync_data(dfw, &opt);
     }
 
     print_finish_info(opt);
