@@ -31,6 +31,7 @@
 #define hfsplus_MAGIC "HFS Plus"
 #define IMAGE_VERSION "0001"
 #define VERSION_SIZE 4
+#define SECTOR_SIZE 512
 
 // Reference: ntfsclone.c
 #define MBYTE (1000 * 1000)
@@ -53,10 +54,12 @@ struct cmd_opt
 {
         int clone;
         int restore;
-//	int dd;
+	int dd;
 	int debug;
         char* source;
         char* target;
+	int overwrite;
+	int rescue;
 };
 typedef struct cmd_opt cmd_opt;
 extern void usage(void);
@@ -72,9 +75,9 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt);
 extern void open_log();
 extern void log_mesg(int lerrno, int lexit, int only_debug, int debug, const char *fmt, ...);
 extern void close_log();
-extern int io_all(int *fd, char *buffer, int count, int do_write, cmd_opt* opt);
+extern int io_all(int *fd, char *buffer, int count, int do_write, cmd_opt *opt);
 extern void sync_data(int fd, cmd_opt* opt);
-
+extern void rescue_sector(int *fd, char *buff, cmd_opt *opt);
 /**
  * for restore used functions
  * restore_image_hdr	- get image_head from image file 
@@ -113,11 +116,17 @@ extern int open_target(char* target, cmd_opt* opt);
 /// check partition size
 extern void check_size(int* ret, unsigned long long size);
 
+/// check free space
+extern void check_free_space(int* ret, unsigned long long size);
+
+/// check free memory size
+extern int check_mem_size(image_head image_hdr, cmd_opt opt, unsigned long long *mem_size);
+
 /// generate crc32 code
 extern unsigned long crc32(unsigned long crc, char *buf, int size);
 
 /// print image_head
 extern void print_image_hdr_info(image_head image_hdr, cmd_opt opt);
-
+extern void print_opt(cmd_opt opt);
 /// print finish mesg
 extern void print_finish_info(cmd_opt opt);
